--[[
Interface: 1.6.0.0 b9173

Copyright (C) GtX (Andy), 2020

Author: GtX | Andy
Date: 18.09.2020
Version: 1.0.0.0

Contact:
https://forum.giants-software.com
https://github.com/GtX-Andy

Important:
Not to be added to any mods / maps or modified from its current release form.
No changes are to be made to this script without permission from GtX | Andy

Darf nicht zu Mods / Maps hinzugefügt oder von der aktuellen Release-Form geändert werden.
An diesem Skript dürfen ohne Genehmigung von GtX | Andy keine Änderungen vorgenommen werden
]]


DynamicWaypoint = {}

local DynamicWaypoint_mt = Class(DynamicWaypoint)

function DynamicWaypoint:new(customEnvironment, baseDirectory, buildId, versionString, mission, i18n, gui)
    local self = setmetatable({}, DynamicWaypoint_mt)

    self.isServer = g_server ~= nil
    self.isClient = g_client ~= nil

    self.buildId = buildId
    self.versionString = versionString

    self.customEnvironment = customEnvironment
    self.baseDirectory = baseDirectory

    if mission ~= nil and mission.inGameMenu ~= nil then
        self.pageMapOverview = mission.inGameMenu.pageMapOverview
    end

    self.mapHotspot = nil
    self.pageMapOverviewUpdated = false

    return self
end

function DynamicWaypoint:load()
    if self.pageMapOverview == nil or self.pageMapOverview.ingameMap == nil or self.pageMapOverviewUpdated then
        g_logManager:error("  [DynamicWaypoint]  GUI frame 'pageMapOverview' could no be found at 'inGameMenu'!")

        return false
    end

    -- Initialise Hotspot
    self:getMapHotspot()

    -- Create the new placeholder and move positions
    local mapWaypointGlyphPlaceholder = self.pageMapOverview.mapZoomGlyphPlaceholder:clone(self.pageMapOverview)
    self.pageMapOverview.mapZoomGlyphPlaceholder.parent:addElement(mapWaypointGlyphPlaceholder)

    local posY = self.pageMapOverview.mapMoveGlyphPlaceholder.position[2]

    self.pageMapOverview.buttonSwitchMapMode:move(0, posY)
    self.pageMapOverview.mapMoveGlyphPlaceholder:move(0, posY)
    self.pageMapOverview.mapZoomGlyphPlaceholder:move(0, posY)

    self.pageMapOverview.mapWaypointGlyphOffsetPos = -posY
    self.pageMapOverview.mapWaypointGlyphPlaceholder = mapWaypointGlyphPlaceholder

    -- Create the new glyph
    local screenValueX = InGameMenuMapFrame.GLYPH_SIZE[1] or 36
    local screenValueY = InGameMenuMapFrame.GLYPH_SIZE[2] or 36
    local iconWidth, iconHeight = getNormalizedScreenValues(screenValueX, screenValueY)

    local waypointInputGlyph = InputGlyphElement:new(self.pageMapOverview.inputDisplayManager, iconWidth, iconHeight)

    waypointInputGlyph:setBold(true)
    waypointInputGlyph:setScale(1, 1)
    waypointInputGlyph:setUpperCase(true)
    waypointInputGlyph:setKeyboardGlyphColor(InGameMenuMapFrame.GLYPH_COLOR)
    waypointInputGlyph:setAlignment(Overlay.ALIGN_VERTICAL_MIDDLE, Overlay.ALIGN_HORIZONTAL_RIGHT)

    self.pageMapOverview.waypointInputGlyph = waypointInputGlyph

    -- Function changes
    self.pageMapOverview.updateInputGlyphs = Utils.prependedFunction(self.pageMapOverview.updateInputGlyphs, self.inj_InGameMenuMapFrame_updateInputGlyphs)
    self.pageMapOverview.onClickTagPlace = Utils.appendedFunction(self.pageMapOverview.onClickTagPlace, self.inj_InGameMenuMapFrame_onClickTagPlace)
    self.pageMapOverview.showContextMarker = Utils.overwrittenFunction(self.pageMapOverview.showContextMarker, self.inj_InGameMenuMapFrame_showContextMarker)
    self.pageMapOverview.delete = Utils.appendedFunction(self.pageMapOverview.delete, self.inj_InGameMenuMapFrame_delete)
    self.pageMapOverview.draw = Utils.appendedFunction(self.pageMapOverview.draw, self.inj_InGameMenuMapFrame_draw)
    self.pageMapOverview.update = Utils.appendedFunction(self.pageMapOverview.update, self.inj_InGameMenuMapFrame_update)

    self.pageMapOverview.ingameMap.mouseEvent = Utils.overwrittenFunction(self.pageMapOverview.ingameMap.mouseEvent, self.inj_IngameMapElement_mouseEvent)
    self.pageMapOverview.ingameMap.onAccept = Utils.appendedFunction(self.pageMapOverview.ingameMap.onAccept, self.inj_IngameMapElement_onAccept)
    self.pageMapOverview.ingameMap.onClickMap = Utils.prependedFunction(self.pageMapOverview.ingameMap.onClickMap, self.inj_IngameMapElement_onClickMap)

    BaseMission.setMapTargetHotspot = Utils.overwrittenFunction(BaseMission.setMapTargetHotspot, self.inj_BaseMission_setMapTargetHotspot)

    self.pageMapOverviewUpdated = true

    return true
end

function DynamicWaypoint:getMapHotspot()
    if self.mapHotspot == nil then
        local mapHotspot = MapHotspot:new(g_i18n:getText("dynamicWaypointGUI_waypoint", self.customEnvironment), MapHotspot.CATEGORY_DEFAULT)

        mapHotspot:setImage(nil, getNormalizedUVs(MapHotspot.UV.HIGHLIGHT_MARKER), {0.27, 0.64, 0.08, 1})

        mapHotspot:setPersistent(true)
        mapHotspot:setBlinking(true)

        mapHotspot:setRenderLast(true)
        mapHotspot:setHasDetails(true)

        mapHotspot:setOwnerFarmId(AccessHandler.EVERYONE)

        mapHotspot.enabled = false

        mapHotspot.isDynamicWaypoint = true
        mapHotspot.addedToHotspots = false

        self.mapHotspot = mapHotspot
    end

    return self.mapHotspot
end

function DynamicWaypoint:delete()
    if self.mapHotspot ~= nil then
        g_currentMission:removeMapHotspot(self.mapHotspot)

        self.mapHotspot.addedToHotspots = false
        self.mapHotspot:delete()

        self.mapHotspot = nil
    end
end

function DynamicWaypoint.inj_InGameMenuMapFrame_delete(frame)
    if frame.waypointInputGlyph ~= nil then
        frame.waypointInputGlyph:delete()
    end

    if frame.mapWaypointGlyphPlaceholder ~= nil then
        if frame.buttonSwitchMapMode ~= nil and frame.mapMoveGlyphPlaceholder ~= nil and frame.mapZoomGlyphPlaceholder ~= nil then
            local posY = frame.mapWaypointGlyphOffsetPos

            if posY == nil then
                posY = frame.mapMoveGlyphPlaceholder.position[2]
            end

            frame.buttonSwitchMapMode:move(0, posY)
            frame.mapMoveGlyphPlaceholder:move(0, posY)
            frame.mapZoomGlyphPlaceholder:move(0, posY)
        end

        frame.mapWaypointGlyphPlaceholder:unlinkElement()
        frame.mapWaypointGlyphPlaceholder:delete()

        frame.mapWaypointGlyphPlaceholder = nil
    end

    frame.lastSelectedHotspot = nil
    frame.dynamicWaypointTexts = nil
end

function DynamicWaypoint.inj_InGameMenuMapFrame_draw(frame)
    if frame.waypointInputGlyph ~= nil then
        frame.waypointInputGlyph:draw()
    end
end

function DynamicWaypoint.inj_InGameMenuMapFrame_update(frame, dt)
    if frame.waypointInputGlyph ~= nil and frame.ingameMap.mapZoom ~= frame.ingameMapLastMapZoom then
        local visible = frame.mode == InGameMenuMapFrame.MODE_OVERVIEW and frame.ingameMap.mapZoom > 0.5

        if visible and frame.ingameMap.inputMode == GS_INPUT_HELP_MODE_GAMEPAD then
            visible = frame.ingameMap.target.currentHotspot == nil
        end

        frame.ingameMapLastMapZoom = frame.ingameMap.mapZoom
        frame.waypointInputGlyph:setVisible(visible)
    end
end

function DynamicWaypoint.inj_InGameMenuMapFrame_updateInputGlyphs(frame)
    if frame.mapWaypointGlyphPlaceholder ~= nil and frame.waypointInputGlyph ~= nil then
        if frame.dynamicWaypointTexts == nil then
            frame.dynamicWaypointTexts = {
                buttonSelectText = frame.l10n:getText("button_select"),
                setWaypointText = frame.l10n:getText("dynamicWaypointGUI_setWaypoint")
            }
        end

        local inputAction = InputAction.MOUSE_ALT_COMMAND2_BUTTON
        local visible = frame.mode == InGameMenuMapFrame.MODE_OVERVIEW and frame.ingameMap.mapZoom > 0.5

        if frame.lastInputHelpMode == GS_INPUT_HELP_MODE_GAMEPAD then
            inputAction = InputAction.INGAMEMAP_ACCEPT
            visible = visible and frame.currentHotspot == nil
        end

        frame.buttonSelectIngame:setText(frame.dynamicWaypointTexts.buttonSelectText)
        frame:updateInputGlyphTransform(frame.mapWaypointGlyphPlaceholder, frame.waypointInputGlyph)
        frame.waypointInputGlyph:setActions({inputAction}, frame.dynamicWaypointTexts.setWaypointText, frame.glyphTextSize, true, true)
        frame.waypointInputGlyph:setVisible(visible)
    end
end

function DynamicWaypoint.inj_InGameMenuMapFrame_showContextMarker(frame, superFunc, active, removeMarker)
    if frame.currentHotspot ~= nil and frame.currentHotspot.isDynamicWaypoint then
        frame.buttonSetMarker:setText(frame.l10n:getText("dynamicWaypointGUI_removeWaypoint"))
    else
        superFunc(frame, active, removeMarker)
    end
end

function DynamicWaypoint.inj_InGameMenuMapFrame_onClickTagPlace(frame)
    if frame.lastInputHelpMode == GS_INPUT_HELP_MODE_GAMEPAD then
        frame.ingameMap.lastSelectedHotspot = frame.currentHotspot
        frame:updateInputGlyphs()
    end
end

function DynamicWaypoint.inj_IngameMapElement_mouseEvent(element, superFunc, posX, posY, isDown, isUp, button, eventUsed)
    local eventUsed = superFunc(element, posX, posY, isDown, isUp, button, eventUsed)

    if element.mapZoom <= 0.5 or g_currentMission == nil or element.target == nil or element.target.mode ~= InGameMenuMapFrame.MODE_OVERVIEW then
        return eventUsed
    end

    if g_dynamicWaypoint ~= nil and not eventUsed and element:getIsActive() then
        local mapHotspot = g_dynamicWaypoint:getMapHotspot()

        if mapHotspot ~= nil then
            if isUp and button == Input.MOUSE_BUTTON_RIGHT then

                if not element.mouseDown and not element.hasDragged then
                    local localPosX, localPosY = element:getLocalPosition(posX, posY)
                    local worldPosX, worldPosZ = element:localToWorldPos(localPosX, localPosY)

                    if not mapHotspot.addedToHotspots then
                        mapHotspot.addedToHotspots = true
                        g_currentMission:addMapHotspot(mapHotspot)
                    end

                    mapHotspot:setWorldPosition(worldPosX, worldPosZ)
                    mapHotspot.enabled = true

                    if element.target.currentHotspot ~= mapHotspot then
                        element.target:setMapSelectionItem(mapHotspot)
                    end

                    g_currentMission:setMapTargetHotspot(mapHotspot)

                    eventUsed = true
                end

                element.mouseDown = false
                element.hasDragged = false
            end
        end
    end

    return eventUsed
end

function DynamicWaypoint.inj_IngameMapElement_onClickMap(element, localPosX, localPosY)
    if element.target ~= nil then
        element.lastSelectedHotspot = element.target.currentHotspot
    end
end

function DynamicWaypoint.inj_IngameMapElement_onAccept(element, posX, posY)
    if element.mapZoom > 0.5 and element.cursorElement ~= nil and element.cursorElement:getIsVisible() and element.target ~= nil then
        if element.target.mode == InGameMenuMapFrame.MODE_OVERVIEW then

            if element.target.currentHotspot == nil and element.lastSelectedHotspot == nil and g_dynamicWaypoint ~= nil then
                local mapHotspot = g_dynamicWaypoint:getMapHotspot()

                local localX, localY = element:getLocaPointerTarget()
                local worldPosX, worldPosZ = element:localToWorldPos(localX, localY)

                if not mapHotspot.addedToHotspots then
                    mapHotspot.addedToHotspots = true
                    g_currentMission:addMapHotspot(mapHotspot)
                end

                mapHotspot:setWorldPosition(worldPosX, worldPosZ)
                mapHotspot.enabled = true

                if element.target.currentHotspot ~= mapHotspot then
                    element.target:setMapSelectionItem(mapHotspot)
                end

                g_currentMission:setMapTargetHotspot(mapHotspot)
                g_currentMission.disableMapTargetHotspotHiding = false
            end

            if element.target ~= nil then
                element.target:updateInputGlyphs()
            end
        end
    end
end

function DynamicWaypoint.inj_BaseMission_setMapTargetHotspot(mission, superFunc, mapHotspot)
    local hotspotToRemove

    if mission.currentMapTargetHotspot ~= nil and mission.currentMapTargetHotspot.isDynamicWaypoint and mission.currentMapTargetHotspot ~= mapHotspot then
        if g_gui.currentGuiName == "InGameMenu" and g_dynamicWaypoint ~= nil and g_dynamicWaypoint.pageMapOverview ~= nil then
            g_dynamicWaypoint.pageMapOverview:setMapSelectionItem(nil)
        end

        hotspotToRemove = mission.currentMapTargetHotspot
    end

    superFunc(mission, mapHotspot)

    if hotspotToRemove ~= nil then
        mission:removeMapHotspot(hotspotToRemove)
        hotspotToRemove.addedToHotspots = false
    end
end
