--[[
Interface: 1.6.0.0 b9173

Copyright (C) GtX (Andy), 2020

Author: GtX | Andy
Date: 18.09.2020
Version: 1.0.0.0

Contact:
https://forum.giants-software.com
https://github.com/GtX-Andy

Important:
Not to be added to any mods / maps or modified from its current release form.
No changes are to be made to this script without permission from GtX | Andy

Darf nicht zu Mods / Maps hinzugefügt oder von der aktuellen Release-Form geändert werden.
An diesem Skript dürfen ohne Genehmigung von GtX | Andy keine Änderungen vorgenommen werden
]]


local loadError
local dynamicWaypoint

local globalTextsLoaded = false

local buildId = 1
local versionString = "0.0.0.0"

local modName = g_currentModName
local modDirectory = g_currentModDirectory

local function updateGlobalTexts(isLoad, i18n)
    local globalTexts = getfenv(0).g_i18n.texts

    if isLoad then
        if not globalTextsLoaded then
            for key, text in pairs(i18n.texts) do
                if StringUtil.startsWith(key, "dynamicWaypointGUI_") then
                    globalTexts[key] = text
                end
            end

            globalTextsLoaded = true
        end
    else
        if globalTextsLoaded then
            for key, text in pairs(i18n.texts) do
                globalTexts[key] = nil
            end

            globalTextsLoaded = false
        end
    end
end

local function loadMission00Finished(mission, superFunc, node)
    dynamicWaypoint = DynamicWaypoint:new(modName, modDirectory, buildId, versionString, mission, g_i18n, g_gui)

    if dynamicWaypoint ~= nil then
        if dynamicWaypoint:load() then
            updateGlobalTexts(true, g_i18n)
            getfenv(0)["g_dynamicWaypoint"] = dynamicWaypoint
        else
            dynamicWaypoint:delete()
            dynamicWaypoint = nil
        end
    end

    return superFunc(mission, node)
end

local function baseMissionDelete()
    if dynamicWaypoint ~= nil then
        dynamicWaypoint:delete()
    end

    updateGlobalTexts(false, g_i18n)

    getfenv(0)["g_dynamicWaypoint"] = nil

    dynamicWaypoint = nil
    loadError = nil
end

local function initLoadError(name)
    loadError = {}

    loadError.name = name
    loadError.modName = modName
    loadError.startUpdateTime = 2000
    loadError.versionString = versionString

    loadError.update = function(self, dt)
        if self.startUpdateTime > 0 then
            self.startUpdateTime = self.startUpdateTime - dt
        else
            local title = string.format("Dynamic Waypoint - Version %s", self.versionString)
            local text = string.format(g_i18n:getText("dynamicWaypoint_loadError", self.modName), self.modName, self.name)

            if g_dedicatedServerInfo == nil then
                if not g_gui:getIsGuiVisible() then
                    g_gui:showYesNoDialog({
                        title = title,
                        text = text,
                        dialogType = DialogElement.TYPE_LOADING,
                        callback = self.openModHubLink,
                        target = nil,
                        yesText = g_i18n:getText("button_ok"),
                        noText = g_i18n:getText("button_modHubDownload")
                    })
                end
            else
                print("\n" .. text .. "\n    - https://farming-simulator.com/mods.php?lang=en&country=be&title=fs2019&filter=org&org_id=129652&page=0" .. "\n")
                self.openModHubLink(true)
            end
        end
    end

    loadError.openModHubLink = function(ignore)
        if ignore == false then
            local language = g_languageShort
            local link = "mods.php?lang=en&country=be&title=fs2019&filter=org&org_id=129652&page=0"
            if language == "de" or language == "fr" then
                link = "mods.php?lang=" .. language .. "&country=be&title=fs2019&filter=org&org_id=129652&page=0"
            end

            openWebFile(link, "")
        end

        removeModEventListener(loadError)
        loadError = nil
    end

    addModEventListener(loadError)
end

local function validate()
    local mod = g_modManager:getModByName(modName)

    versionString = tostring(mod.version) or versionString

    if mod.modName == "FS19_DynamicWaypoint" or
        mod.modName == "FS19_DynamicWaypoint_update" then

        return mod.author:len() == 3, mod.author
    end

    return false, mod.author
end

local function initDynamicWaypoint()
    if g_dynamicWaypoint ~= nil then
        g_logManager:error("Dynamic Waypoint has already been loaded by '%s'! Please remove duplicate mod '%s'.", tostring(g_dynamicWaypoint.customEnvironment), modName)
        return
    end

    local canLoad, author = validate()

    if canLoad then
        source(modDirectory .. "scripts/DynamicWaypoint.lua")

        Mission00.loadMission00Finished = Utils.overwrittenFunction(Mission00.loadMission00Finished, loadMission00Finished)
        FSBaseMission.delete = Utils.appendedFunction(FSBaseMission.delete, baseMissionDelete)
    else
        initLoadError(author)
    end
end

initDynamicWaypoint()
